@@ -3,9 +3,9 @@ package global
 import (
 	"testing"
 
-	"github.com/dop251/goja"
-	"github.com/dop251/goja_nodejs/console"
-	"github.com/dop251/goja_nodejs/require"
+	"github.com/Mzack9999/goja"
+	"github.com/Mzack9999/goja_nodejs/console"
+	"github.com/Mzack9999/goja_nodejs/require"
 )
 
 func TestScriptsRuntime(t *testing.T) {
