@@ -4,7 +4,9 @@ import (
 	"context"
 	"database/sql"
 	"database/sql/driver"
+	"fmt"
 	"net"
+	"net/url"
 	"time"
 
 	"github.com/lib/pq"
@@ -17,21 +19,33 @@ const (
 )
 
 type pgDial struct {
-	fd *fastdialer.Dialer
+	executionId string
 }
 
 func (p *pgDial) Dial(network, address string) (net.Conn, error) {
-	return p.fd.Dial(context.TODO(), network, address)
+	dialers := protocolstate.GetDialersWithId(p.executionId)
+	if dialers == nil {
+		return nil, fmt.Errorf("dialers not initialized for %s", p.executionId)
+	}
+	return dialers.Fastdialer.Dial(context.TODO(), network, address)
 }
 
 func (p *pgDial) DialTimeout(network, address string, timeout time.Duration) (net.Conn, error) {
+	dialers := protocolstate.GetDialersWithId(p.executionId)
+	if dialers == nil {
+		return nil, fmt.Errorf("dialers not initialized for %s", p.executionId)
+	}
 	ctx, cancel := context.WithTimeoutCause(context.Background(), timeout, fastdialer.ErrDialTimeout)
 	defer cancel()
-	return p.fd.Dial(ctx, network, address)
+	return dialers.Fastdialer.Dial(ctx, network, address)
 }
 
 func (p *pgDial) DialContext(ctx context.Context, network, address string) (net.Conn, error) {
-	return p.fd.Dial(ctx, network, address)
+	dialers := protocolstate.GetDialersWithId(p.executionId)
+	if dialers == nil {
+		return nil, fmt.Errorf("dialers not initialized for %s", p.executionId)
+	}
+	return dialers.Fastdialer.Dial(ctx, network, address)
 }
 
 // Unfortunately lib/pq does not provide easy to customize or
@@ -45,7 +59,18 @@ type PgDriver struct{}
 // Most users should only use it through database/sql package from the standard
 // library.
 func (d PgDriver) Open(name string) (driver.Conn, error) {
-	return pq.DialOpen(&pgDial{fd: protocolstate.Dialer}, name)
+	// Parse the connection string to get executionId
+	u, err := url.Parse(name)
+	if err != nil {
+		return nil, fmt.Errorf("invalid connection string: %v", err)
+	}
+	values := u.Query()
+	executionId := values.Get("executionId")
+	// Remove executionId from the connection string
+	values.Del("executionId")
+	u.RawQuery = values.Encode()
+
+	return pq.DialOpen(&pgDial{executionId: executionId}, u.String())
 }
 
 func init() {
